
#ifndef __HID__H_
#define __HID__H_

#include "usb.h"

#define MSD_DATA_SIZE        64

// The USB calls generic class functions, which actually are HID
#define ClassInitEndpoint MSDInitEndpoint
#define ProcessClassRequest ProcessMSDRequest
#define ClassOutNotify MSDOutNotify

void MSDInitEndpoint(void);
char ProcessMSDRequest(void);
void MSDOutNotify(void);
void ClassTransfer(void);

extern volatile unsigned char ep1_in_buffer[MSD_DATA_SIZE];
extern volatile unsigned char ep1_out_buffer[MSD_DATA_SIZE];

char GetInputReport(unsigned char reportID);
char GetFeatureReport(unsigned char reportID);
char SetupOutputReport(unsigned char reportID);
char SetupFeatureReport(unsigned char reportID);
void SetOutputReport(unsigned char reportID);
void SetFeatureReport(unsigned char reportID);

#define BIG_ENDIAN(x) ( ((x&0xFF)<<8) | ((x>>8)&0xFF) )
#define BIG_ENDIAN4(x)  (((unsigned long)x&0xFF)<<24) |  \
			(((unsigned long)x&0xFF00)<<8) | \
			(((unsigned long)x&0xFF0000)>>8) | \
			(((unsigned long)x&0xFF000000)>>24)

#define MSD_RESET 0xff
#define GET_MAX_LUN 0xfe

#define MSD_STATE_WAIT     0
#define MSD_STATE_IN       1
#define MSD_STATE_OUT      2
#define MSD_STATE_DATA_IN  3
#define MSD_STATE_DATA_OUT 4
#define MSD_STATE_CSW      5

// SCSI Transparent Command Set
#define MSD_COMM_INQUIRY			0x12
#define MSD_COMM_READ_FORMAT_CAPACITY		0x23
#define MSD_COMM_READ_CAPACITY			0x25
#define MSD_COMM_READ_10			0x28
#define MSD_COMM_WRITE_10			0x2a
#define MSD_COMM_REQUEST_SENSE			0x03
#define MSD_COMM_MODE_SENSE			0x1a
#define MSD_COMM_MODE_SENSE_5a			0x5a
#define MSD_COMM_PREVENT_ALLOW_MEDIUM_REMOVAL	0x1e
#define MSD_COMM_TEST_UNIT_READY		0x00
#define MSD_COMM_VERIFY				0x2f
#define MSD_COMM_STOP_START			0x1b
#define MSD_COMM_LIST_LUNS			0xa0
#define MSD_COMM_GET_CONFIGURATION		0x46
#define MSD_COMM_GET_TOC			0x43
#define MSD_COMM_READ_DISC_INFO			0x51
#define MSD_COMM_REPORT_KEY			0xa4

typedef struct {
	unsigned char peripheral;
	unsigned char RMB;
	unsigned char version;
	unsigned char response;
	unsigned char add_length;
	unsigned char rs[3];
	unsigned char vendorID[8];
	unsigned char productID[16];
	unsigned char productRev[4];
} InquiryStruct;

typedef struct {
	unsigned char signature[4];
	unsigned char tag[4];
	unsigned long residue;
	unsigned char status;
} CSWStruct; // 13 bytes

typedef struct {
	unsigned short ModeDataLen;
	unsigned char MTC;
	unsigned char r[5];
} PageHeaders; // 4 bytes

typedef struct {
	unsigned char page_code;
	unsigned char page_length;
	unsigned char params;
	unsigned char readrc;
	unsigned char r1[4];
	unsigned char writerc;
	unsigned char r2[3];
} ErrRecPage; // 12 bytes

typedef struct {
	unsigned char page_code;
	unsigned char page_length;
	unsigned short transfer_rate;
	unsigned char numh;
	unsigned char sectrack;
	unsigned short bytes_per_sector;
	unsigned short num_cyl;
	unsigned char r1[9];
	unsigned char ond;
	unsigned char offd;
	unsigned char r2[7];
	unsigned short rrate;
	unsigned char r3[2];
} FlexDiskPage; // 32 bytes

typedef struct {
	unsigned char page_code;
	unsigned char page_length;
	unsigned char z1;
	unsigned char tlun;
	unsigned char r1[8];
} AccessCapPage; // 12 bytes

typedef struct {
	unsigned char page_code;
	unsigned char page_length;
	unsigned char r1;
	unsigned char inact;
	unsigned char r2[4];
} TimerPage; // 8 bytes

typedef struct {
	PageHeaders header; // 4
	FlexDiskPage p2;    // 32
} MSDPages; // 36 bytes

typedef struct {
	unsigned char sig[4];
	unsigned char tag[4];
	unsigned long transfer_len;
	unsigned char flags;
	unsigned char lun;
	unsigned char cbw_len;
} CBWStruct;

typedef struct {
	unsigned char v_err;
	unsigned char rsvd1;
	unsigned char sensekey;
	unsigned long info;
	unsigned char add_senselen;
	unsigned long rsvd2;
	unsigned char sense_code;
	unsigned char sense_qualifier;
	unsigned long rsvd3;
} StdSenseData;

typedef struct {
	unsigned long data_len;
	unsigned short reserved;
	unsigned short curr_profile;
} FeatureHeader;

typedef struct {
	unsigned short feature_code;
	unsigned char flags;
	unsigned add_len;
	
} FeatureDescriptor;

#endif

